import React, { useState, useEffect } from 'react';
import { LampCirclePicker } from '@ray-js/components-ty-lamp';
import throttle from 'lodash/throttle';
import { utils } from '@ray-js/panel-sdk';
import { getElementById } from '@ray-js/api';
import { hsvToDpdata, dpdataToHsv, temperatureToRgb } from '@/utils';
import { useAtomValue, useSetAtom } from 'jotai';
import { ColorData, colorDataAtom, selectColorDataAtom } from '@/atoms';
import { View, Text, Slider, Image, Modal } from '@ray-js/ray';
import styles from './index.module.less';
import { Props } from './index.type';
import imgs from '../../res';

type colorItem = Record<string, any>;

const HubCircle: React.FC<Props> = props => {
  const dpState = props.DpStateData;
  const codeMap = props.DpCodesMap;

  const setColorStateAtom = useSetAtom(colorDataAtom);
  const colorStateInAtom = useAtomValue(selectColorDataAtom);

  const { hsv2rgbString, rgb2hex, rgb2hsv } = utils;
  // document.getElementById;
  const dataSourse = {
    isDrag: false,

    angleLimit: 135, // 角度限制
    angleValue: 0, // 角度信息 0 ~ 360
    tempValue: [2700, 6500], // 色温范围

    containerWidth: 400,

    containerStyle: {
      width: '200px',
      height: '200px',
      backgroundSize: '200px 200px',
    },

    circleStyle: {
      width: '100px',
      height: '100px',
      left: '50px',
      top: '50px',
    },

    containerOffset: {
      // 容器位置
      x: 100,
      y: 150,
    },

    mouse_offset: { x: 0, y: 0 },

    circle_r: 100, // 圆形外半径(容器宽度/2)
    circle_b: 42, // 圆形边距
    bar_r: 21, // 小球半径(圆形边距/2)
    bar_c_r: 100 - 21, // 小球轨迹半径(圆形边距/2)
  };

  const [addFlag, setAddFlag] = useState(true);
  const [modalShow, setModalShow] = useState(false);
  const [brightValue, setbrightValue] = useState(dpState.bright_value);

  const [colorArr, setColorArr] = useState<colorItem[]>([
    { temp_value: 100, bright_value: 100, color: '#CEEDFE', active: false },
    { temp_value: 200, bright_value: 200, color: '#F5FCFF', active: false },
    { temp_value: 300, bright_value: 300, color: '#FFE2A4', active: false },
  ]);

  const getTempshowByDpTemp = (dpTemp: number) => {
    const start = dataSourse.tempValue[0];
    const end = dataSourse.tempValue[1];
    const tempValue = ((end - start) / 1000) * dpTemp + start;
    return tempValue;
  };

  const getColor = (bright_value: number, KL = (6500 - 2700) / 2 + 2700) => {
    const colorRgb = temperatureToRgb(KL);
    const colorHsv = rgb2hsv(colorRgb[0], colorRgb[1], colorRgb[2]);
    const color = hsv2rgbString(colorHsv[0], colorHsv[1], colorHsv[2], bright_value / 1000);
    return color;
  };

  const [nowColor, setnowColor] = useState(
    getColor(dpState.bright_value, getTempshowByDpTemp(dpState.temp_value))
  );

  const [dataState, setDataState] = useState({
    angleValue: 0, // 角度值 0 ~ 360
    angleShow: 0, // 角度展示 0 ~ 180
    tempshow: (6500 - 2700) / 2 + 2700, // 色温展示
  });

  // useEffect(() => {
  //   const kl = getTempshowByDpTemp(dpState.temp_value);
  //   const color = getColor(dpState.bright_value, kl);
  //   setDataState({
  //     ...dataState,
  //     nowColor: color,
  //     tempshow: kl,
  //   });
  // }, [dpState.temp_value]);
  // 442
  const circleLocationOrigin = (tempValue: number) => {
    let rad;
    let left = 79;
    let top = 0;
    if (tempValue > 500) {
      rad = ((tempValue - 500) * (dataSourse.angleLimit / 500) * Math.PI) / 180;
      const y = dataSourse.bar_c_r * Math.cos(rad);
      const x = dataSourse.bar_c_r * Math.sin(rad);
      left += x;
      top = dataSourse.circle_r - y - dataSourse.bar_r;
    }

    if (tempValue < 500) {
      rad = ((500 - tempValue) * (dataSourse.angleLimit / 500) * Math.PI) / 180;
      const x = dataSourse.bar_c_r * Math.sin(rad);
      const y = dataSourse.bar_c_r * Math.cos(rad);
      left -= x;
      top = dataSourse.circle_r - y - dataSourse.bar_r;
    }
    return {
      left,
      top,
    };
  };

  const [circleLocation, setCircleLocation] = useState({
    // 小球位置
    left: circleLocationOrigin(dpState.temp_value).left,
    top: circleLocationOrigin(dpState.temp_value).top,
  });

  const toParseInt = (value: number): number => {
    return Math.floor(value);
  };

  const handleCirleClick = (e: any, item: colorItem): void => {
    e.origin.stopPropagation();

    const index = colorArr.findIndex(el => {
      return item.bright_value === el.bright_value && el.temp_value === item.temp_value;
    });

    if (item.active) {
      colorArr.splice(index, 1);
      if (colorArr.length < 6) {
        setAddFlag(true);
      }
    } else {
      for (let i = 0; i < colorArr.length; i++) {
        colorArr[i].active = false;
      }
      colorArr[index].active = true;
    }
    const value = [...colorArr];
    setColorArr(value);
  };

  const handleCircleAdd = () => {
    if (colorArr.length > 5) {
      setAddFlag(false);
      return;
    }

    const color = getColor(dpState.bright_value, dataState.tempshow);
    setnowColor(color);

    const item = {
      temp_value: dpState.temp_value,
      bright_value: dpState.bright_value,
      color: nowColor,
      active: false,
    };
    const flag = colorArr.some((el: colorItem) => {
      return item.bright_value === el.bright_value && item.temp_value === el.temp_value;
    });

    if (flag) {
      setModalShow(true);
      return;
    }

    colorArr.push(item);
    const value = [...colorArr];
    setColorArr(value);
  };

  const handleBarTouch = (e: any) => {
    e.origin.stopPropagation();
    setColorStateAtom({ ifDrag: true });
    // 移动距离：小球的坐标位置-容器的偏移位置-小球相对容器的偏移位置-小球半径
    const mouse_offset_x =
      e.origin.touches[0].clientX -
      dataSourse.containerOffset.x -
      e.origin.target.offsetLeft -
      dataSourse.bar_r;

    const mouse_offset_y = -(
      e.origin.touches[0].clientY -
      dataSourse.containerOffset.y -
      e.origin.target.offsetTop -
      dataSourse.bar_r
    );
    dataSourse.mouse_offset.x = mouse_offset_x;
    dataSourse.mouse_offset.y = mouse_offset_y;
  };

  const handleBarMove = throttle((e: any) => {
    e.origin.stopPropagation();
    const event_offset_x =
      e.origin.touches[0].clientX -
      dataSourse.mouse_offset.x -
      dataSourse.containerOffset.x -
      dataSourse.circle_r;
    const event_offset_y = -(
      e.origin.touches[0].clientY -
      dataSourse.mouse_offset.y -
      dataSourse.containerOffset.y -
      dataSourse.circle_r
    );

    const radian = Math.atan2(event_offset_y, event_offset_x);
    const x = Math.cos(radian) * (dataSourse.circle_r - dataSourse.circle_b / 2);
    const y = Math.sin(radian) * (dataSourse.circle_r - dataSourse.circle_b / 2);

    const left = x - dataSourse.bar_r + dataSourse.circle_r;
    const top = dataSourse.circle_r - (y + dataSourse.bar_r);

    updateContent(radian);

    if (!dataSourse.isDrag) {
      return;
    }
    setCircleLocation({
      ...circleLocation,
      left,
      top,
    });
  }, 50);

  const handleBarEnd = () => {
    setColorStateAtom({ ifDrag: false });
    const dpValue = getTempDpValue(dataState.angleValue, dataState.angleShow);
    const color = getColor(dpState.bright_value, dataState.tempshow);
    setnowColor(color);
    props.onSetDpState('temp_value', dpValue);
  };

  const updateContent = (radian: number) => {
    let angle = radian * (180 / Math.PI); // -180 ~ 180
    if (angle >= -180 && angle <= 90) {
      angle = 90 - angle;
    } else {
      angle = 360 - (angle - 90);
    }
    // 0 ~ 360
    const angleValue = Math.floor(angle);

    // 0 ~ 180
    const angleShow = angleValue > 180 ? 360 - angleValue : angleValue;

    const { angleLimit } = dataSourse;

    const tempshow = getTempshow(angleValue);

    if (angleShow > angleLimit) {
      dataSourse.isDrag = false;
      return;
    }
    if (!dataSourse.isDrag) {
      dataSourse.isDrag = true;
    }

    setDataState({
      ...dataState,
      angleValue,
      angleShow,
      tempshow,
    });
  };

  const getTempshow = (angleValue: number): number => {
    const { angleLimit } = dataSourse;
    const angleShow = angleValue > 180 ? 360 - angleValue : angleValue;
    const tempStart = dataSourse.tempValue[0];
    const tempEnd = dataSourse.tempValue[1];

    let tempshow = (tempEnd - tempStart) / 2 + tempStart; // 中间值 角度对应0
    if (angleValue < 180) {
      tempshow += ((tempEnd - tempshow) / angleLimit) * angleShow;
    }

    if (angleValue > 180) {
      tempshow -= ((tempEnd - tempshow) / angleLimit) * angleShow;
    }

    tempshow = Math.floor(tempshow);
    return tempshow;
  };

  const getTempDpValue = (angleValue, angleShow) => {
    const { angleLimit } = dataSourse;
    let tempDpshow = 500; // 中间值 角度对应0
    if (angleValue < 180) {
      tempDpshow += (500 / angleLimit) * angleShow;
    }

    if (angleValue > 180) {
      tempDpshow -= (500 / angleLimit) * angleShow;
    }
    return Math.floor(tempDpshow);
  };

  const handleModalChange = (flag = false) => {
    setModalShow(flag);
  };

  const handleSlider = (type, e) => {
    const bright_value = e.value;
    props.onSetDpState('bright_value', bright_value);
  };

  const handleSlidering = (type, e) => {
    const bright_value = e.value;
    const color = getColor(dpState.bright_value, dataState.tempshow);

    setnowColor(color);
    setbrightValue(bright_value);
  };

  // const [temperature, setTemperature] = useState(dpState.temp_value);
  // const handleMove = (v: number) => {
  //   setTemperature(v);
  // };
  // const handleEnd = (v: number) => {
  //   setTemperature(v);
  // };

  return (
    <View className={styles.box}>
      <View className={styles.header}>
        <View className={styles.circleContent} id="circleContent" style={dataSourse.containerStyle}>
          <View className={styles.circle} style={dataSourse.circleStyle}>
            {dataState.tempshow}K
          </View>
          <View
            className={styles.circleBar}
            style={{
              left: `${circleLocation.left}px`,
              top: `${circleLocation.top}px`,
              width: `${dataSourse.bar_r * 2}px`,
              height: `${dataSourse.bar_r * 2}px`,
              backgroundColor: nowColor,
            }}
            onTouchStart={handleBarTouch}
            onTouchMove={handleBarMove}
            onTouchEnd={handleBarEnd}
          />
        </View>
        {/* <LampCirclePicker
          canvasId="whiteRing"
          temperature={temperature}
          onTouchMove={handleMove}
          onTouchEnd={handleEnd}
        /> */}
      </View>
      <View className={styles.panel}>
        <View className={styles.configList}>
          <Image src={imgs.icon_ld} className={styles.icoImg} />
          <View className={styles.slider}>
            <Slider
              activeColor="#fff"
              backgroundColor="#333333"
              min={10}
              max={1000}
              step={1}
              value={brightValue}
              onChange={e => handleSlider('bright_value', e)}
              onChanging={e => handleSlidering('bright_value', e)}
            />
          </View>
          <Text className={styles.SliderValue}>{toParseInt((brightValue / 1000) * 100)}%</Text>
        </View>
        <View className={styles.configList}>
          {colorArr.map(item => (
            <View
              className={styles.colorCircle}
              style={{ backgroundColor: item.color, borderColor: item.active ? 'red' : 'white' }}
              key={item.color}
              onClick={e => handleCirleClick(e, item)}
            >
              {item.active && <View className={styles.cutBar}>-</View>}
            </View>
          ))}
          {addFlag && (
            <View className={`${styles.colorCircle} ${styles.addBar}`} onClick={handleCircleAdd}>
              <View className={styles.addStyle}>+</View>
            </View>
          )}
        </View>
      </View>
      <Modal
        show={modalShow}
        position="center"
        overlay
        onClickOverlay={() => {
          handleModalChange(false);
        }}
      >
        <View className={styles.modal}>
          <View>已有相同颜色</View>
          <View
            onClick={() => {
              handleModalChange(false);
            }}
          >
            确定
          </View>
        </View>
      </Modal>
    </View>
  );
};

export default HubCircle;
