import React, { useState } from 'react';
import { utils } from '@ray-js/panel-sdk';
import { hsvToDpdata, dpdataToHsv } from '@/utils';
import { View, Text, Slider, Image, Modal } from '@ray-js/ray';
import styles from './index.module.less';
import { Props } from './index.type';
import imgs from '../../res';
import HubColorHeader from '../HubColorHeader';

type colorItem = Record<string, any>;

const HubCircle: React.FC<Props> = props => {
  const dpState = props.DpStateData;
  const codeMap = props.DpCodesMap;

  const { hsv2rgbString } = utils;

  const Hue = dpdataToHsv(dpState.colour_data)[0];
  const Temp_value = dpdataToHsv(dpState.colour_data)[1];
  const Bright_value = dpdataToHsv(dpState.colour_data)[2];

  const hueToColor = (h: number, s = 100, v = 100, a = 1) => {
    return hsv2rgbString(h, s, v, a);
  };

  const [addFlag, setAddFlag] = useState(true);
  const [modalShow, setModalShow] = useState(false);

  const [colorArr, setColorArr] = useState<colorItem[]>([
    { hue: 0, temp_value: 1000, bright_value: 1000, color: hueToColor(0), active: false },
    { hue: 90, temp_value: 1000, bright_value: 1000, color: hueToColor(90), active: false },
    { hue: 180, temp_value: 1000, bright_value: 1000, color: hueToColor(180), active: false },
  ]);

  const [dataState, setDataState] = useState({
    hue: Hue, // 角度展示
    temp_value: Temp_value,
    bright_value: Bright_value,
    nowColor: hueToColor(Hue), // 当前颜色
  });

  const toParseInt = (value: number): number => {
    return Math.floor(value);
  };

  const handleCirleClick = (e: any, item: colorItem): void => {
    e.origin.stopPropagation();
    const dpValue = hsvToDpdata(dataState.hue, dataState.temp_value, dataState.bright_value);
    props.onSetDpState('colour_data', dpValue);
    const index = colorArr.findIndex(el => {
      return (
        item.hue === el.hue &&
        item.bright_value === el.bright_value &&
        el.temp_value === item.temp_value
      );
    });

    if (item.active) {
      colorArr.splice(index, 1);
      if (colorArr.length < 6) {
        setAddFlag(true);
      }
    } else {
      for (let i = 0; i < colorArr.length; i++) {
        colorArr[i].active = false;
      }
      colorArr[index].active = true;
    }
    const value = [...colorArr];
    setColorArr(value);
  };

  const handleCircleAdd = () => {
    if (colorArr.length > 5) {
      setAddFlag(false);
      return;
    }
    const h = dataState.hue;
    const nowColor = hueToColor(h, dataState.temp_value / 10, dataState.bright_value / 10);

    setDataState({
      ...dataState,
      nowColor,
    });

    const item = {
      hue: h,
      temp_value: dataState.temp_value,
      bright_value: dataState.bright_value,
      color: nowColor,
      active: false,
    };
    const flag = colorArr.some((el: colorItem) => {
      return (
        item.hue === el.hue &&
        item.bright_value === el.bright_value &&
        item.temp_value === el.temp_value
      );
    });

    if (flag) {
      setModalShow(true);
      return;
    }

    colorArr.push(item);
    const value = [...colorArr];
    setColorArr(value);
  };

  const handleStateChange = (type: string, e: any, ifEnd?: string) => {
    if (type === 'temp_value') {
      const temp_value = e.value;
      const dpValue = hsvToDpdata(dataState.hue, temp_value, dataState.bright_value);
      if (ifEnd) {
        props.onSetDpState('colour_data', dpValue);
        return;
      }
      setDataState({
        ...dataState,
        temp_value,
      });
    }
    if (type === 'bright_value') {
      const bright_value = e.value;
      const dpValue = hsvToDpdata(dataState.hue, dataState.temp_value, bright_value);
      if (ifEnd) {
        props.onSetDpState('colour_data', dpValue);
        return;
      }
      setDataState({
        ...dataState,
        bright_value,
      });
    }
  };

  const handleModalChange = (flag = false) => {
    setModalShow(flag);
  };

  const handleBarEnd = (hue: number) => {
    const dpValue = hsvToDpdata(hue, dataState.temp_value, dataState.bright_value);
    props.onSetDpState('colour_data', dpValue);
  };

  return (
    <View className={styles.box}>
      <View className={styles.header}>
        <HubColorHeader
          hue={Hue}
          temp={dataState.temp_value}
          bright={dataState.bright_value}
          onBarMoveEnd={handleBarEnd}
        />
      </View>
      <View className={styles.panel}>
        <View className={styles.configList}>
          <Image src={imgs.icon_bhd} className={styles.icoImg} />
          <View className={styles.slider}>
            <Slider
              activeColor="#fff"
              backgroundColor="#333333"
              min={0}
              max={1000}
              step={1}
              value={dataState.temp_value}
              onChange={e => handleStateChange('temp_value', e, 'end')}
              onChanging={e => handleStateChange('temp_value', e)}
            />
          </View>
          <Text className={styles.SliderValue}>
            {toParseInt((dataState.temp_value / 1000) * 100)}%
          </Text>
        </View>
        <View className={styles.configList}>
          <Image src={imgs.icon_ld} className={styles.icoImg} />
          <View className={styles.slider}>
            <Slider
              activeColor="#fff"
              backgroundColor="#333333"
              min={10}
              max={1000}
              step={1}
              value={dataState.bright_value}
              onChange={e => handleStateChange('bright_value', e, 'end')}
              onChanging={e => handleStateChange('bright_value', e)}
            />
          </View>
          <Text className={styles.SliderValue}>
            {toParseInt((dataState.bright_value / 990) * 100)}%
          </Text>
        </View>
        <View className={styles.configList}>
          {colorArr.map(item => (
            <View
              className={styles.colorCircle}
              style={{ backgroundColor: item.color, borderColor: item.active ? 'red' : 'white' }}
              key={item.color}
              onClick={e => handleCirleClick(e, item)}
            >
              {item.active && <View className={styles.cutBar}>-</View>}
            </View>
          ))}
          {addFlag && (
            <View className={`${styles.colorCircle} ${styles.addBar}`} onClick={handleCircleAdd}>
              <View className={styles.addStyle}>+</View>
            </View>
          )}
        </View>
      </View>
      <Modal
        show={modalShow}
        position="center"
        overlay
        onClickOverlay={() => {
          handleModalChange(false);
        }}
      >
        <View className={styles.modal}>
          <View>已有相同颜色</View>
          <View
            onClick={() => {
              handleModalChange(false);
            }}
          >
            确定
          </View>
        </View>
      </Modal>
    </View>
  );
};

export default HubCircle;
